'''
interrupts.py
Script to show how to use interrupts on the raspberry pi with Sparkfun6DoFIMU
ryan.j.mortenson@gmail.com
November 12th, 2016
https://github.com/sparkfun/SparkFun_6DOF_IMU_Pi_Library

Resources:
cffi==1.9.1
pycparser==2.17
smbus-cffi==0.5.1

Development environment specifics:
python 2.7
vim 7.4
Raspberry Pi 3
Ubuntu 16.04

This code is released under the [MIT License](http:#opensource.org/licenses/MIT).

Please review the LICENSE.md file included with this example. If you have any questions
or concerns with licensing, please contact techsupport@sparkfun.com.

Distributed as-is; no warranty is given.
'''

import argparse
import RPi.GPIO as gpio
import logging
import src.itg3200 as Itg3200
import src.adxl345 as Adxl345
import sys
from time import sleep

# Global variables
log = None
gyro_reads = 0
accel_reads = 0

def adxl_callback(channel):
    '''
    Callback function for interrupt on adxl pin
    '''

    log.debug("Interrupt caught for adxl")

    try:
        # Read each Accel measurement seperately
        log.info("All Accel values read seperately")
        log.info("Accel X Accel {}".format(adxl345.get_x_accel()))
        log.info("Accel Y Accel {}".format(adxl345.get_y_accel()))
        log.info("Accel Z Accel {}".format(adxl345.get_z_accel()))

        # Read all Accel measurements at once
        adxl345.get_all()
        log.info("All Accel values read together")
        log.info("Accel X Accel {}".format(adxl345.x_accel))
        log.info("Accel Y Accel {}".format(adxl345.y_accel))
        log.info("Accel Z Accel {}".format(adxl345.z_accel))

        # Read gyro to reset interrupt
        itg3200.read_byte_data(itg3200.INT_STATUS)
        global accel_reads
        accel_reads += 1
    except:
        pass

def itg_callback(channel):
    '''
    Callback function for interrupt on itg pin
    '''

    log.debug("Interrupt caught for adxl")

    try:
        # Read each Gyro measurement seperately
        log.info("All Gyro values read seperately")
        log.info("Gyro X Axis {}".format(itg3200.get_x_axis()))
        log.info("Gyro Y Axis {}".format(itg3200.get_y_axis()))
        log.info("Gyro Z Axis {}".format(itg3200.get_z_axis()))
        log.info("Gyro Temperature {}".format(itg3200.get_temp()))

        # Read all Gyro measurements at once
        itg3200.get_all()
        log.info("All Gyro values read together")
        log.info("Gyro X Axis {}".format(itg3200.x_axis))
        log.info("Gyro Y Axis {}".format(itg3200.y_axis))
        log.info("Gyro Z Axis {}".format(itg3200.z_axis))
        log.info("Gyro Temperature {}".format(itg3200.temp))

        # Read accel to reset interrupt
        adxl345.get_x_accel()
        global gyro_reads
        gyro_reads += 1
    except:
        pass

if __name__ == "__main__":

    # Setup some arguments for command line execution
    parser = argparse.ArgumentParser()
    parser.add_argument("--i2cbus", "-i", type=int,
                        default=1, help="I2C bus to use default is 1")
    parser.add_argument("--verbosity", "-v", type=str, choices=["DEBUG", "INFO"],
                        default="INFO", help="Repititions default is 1")
    parser.add_argument("--adxl_interrupt", "-a", type=int,
                        default=27, help="Pin adxl interrupt is on")
    parser.add_argument("--itg_interrupt", "-t", type=int,
                        default=17, help="Pin itg interrupt is on")
    args = parser.parse_args()

    # Set logging format
    FORMAT = '[%(asctime)s %(filename)15s line: %(lineno)5s] %(message)s'
    logging.basicConfig(stream=sys.stdout, format=FORMAT, level=getattr(logging, args.verbosity))
    log = logging.getLogger("main")

    # Set gpio mode
    gpio.setmode(gpio.BCM)

    # Setup gpio pins as input and set interrupt
    gpio.setup(args.adxl_interrupt, gpio.IN, pull_up_down=gpio.PUD_UP)
    gpio.add_event_detect(args.adxl_interrupt, gpio.FALLING, callback=adxl_callback, bouncetime=3)
    gpio.setup(args.itg_interrupt, gpio.IN, pull_up_down=gpio.PUD_UP)
    gpio.add_event_detect(args.itg_interrupt, gpio.FALLING, callback=itg_callback, bouncetime=3)

    # Instantiate device classes
    itg3200 = Itg3200.Itg3200(args.i2cbus)
    adxl345 = Adxl345.Adxl345(args.i2cbus)

    # Turn on measurement for adxl345
    adxl345.measurement_on()

    # Setup accel for 10-bit mode with no justification at +/- 2g range
    adxl345.write_byte_data(adxl345.DATA_FORMAT, 32)

    # Setup accel with a data rate of 25 Hz
    adxl345.write_byte_data(adxl345.BW_RATE, 8)

    # Set accel INT_ENABLE to 10000000 which is enable DATA_READY
    # Set accel INT_MAP to 00000000 which is all interrupts on INT1 pin
    adxl345.set_interrupt_mode(128, 0)

    # Setup gyro with a sample rate divider of 255 this combined with
    # bandwidth of 256Hz bandwidth gives a data rate of about 32 Hz
    itg3200.write_byte_data(itg3200.SMPLRT_DIV, 255)

    # Setup gyro for +/-2000 deg/sec with low-pass filter bandwidth of 256 Hz
    itg3200.write_byte_data(itg3200.DLPF_FS, 24)

    # Set gyro interrupt mode to 01110101 which is:
    # Active high, open drain, latch until interrupt cleared, clear on any register read
    # enable when device is ready, enable interrupt when data is ready
    itg3200.set_interrupt_mode(33)

    # Read data to start reset interrupts on devices
    adxl345.get_x_accel()
    itg3200.get_x_axis()

    # Loop until users hits ctrl-c then print number of measurements
    try:
        while(1):
            sleep(.001)
            log.debug("LOOPING")
    except:
        raise
    finally:
        gpio.cleanup()
        log.info("Accel reads: {}".format(accel_reads))
        log.info("Gyro reads: {}".format(gyro_reads))
