'''
protobuf_client.py
Script that registers with protobuf server to receive measurements from Sparkfun6DoF
ryan.j.mortenson@gmail.com
November 12th, 2016
https://github.com/sparkfun/SparkFun_6DOF_IMU_Pi_Library

Resources:
cffi==1.9.1
pycparser==2.17
smbus-cffi==0.5.1
protobuf==2.6.1

Development environment specifics:
python 2.7
vim 7.4
Raspberry Pi 3
Ubuntu 16.04
matplotlib 1.5.3

This code is released under the [MIT License](http:#opensource.org/licenses/MIT).

Please review the LICENSE.md file included with this example. If you have any questions
or concerns with licensing, please contact techsupport@sparkfun.com.

Distributed as-is; no warranty is given.
'''

import argparse
import logging
import matplotlib.pyplot as plt
import socket
import struct
import sys
import traceback
from messages_pb2 import Register, Response, Status
from messages_pb2 import MessageType, ACK, NAK, Generic
from messages_pb2 import ACCELMEASUREMENT, GYROMEASUREMENT, GyroMeasurement, AccelMeasurement
import messages_pb2
from time import sleep

gyro_list = []
accel_list = []

if __name__ == "__main__":

    # Setup some arguments for command line execution
    parser = argparse.ArgumentParser()
    parser.add_argument("--verbosity", "-v", type=str, choices=["DEBUG", "INFO"],
                        default="INFO", help="Repititions default is 1")
    parser.add_argument("--ip_add", "-i", type=str, default="127.0.0.1",
                        help="IP address of raspberry pi with running protobuf server")
    parser.add_argument("--port", "-p", type=int, default="12345",
                        help="PORT of raspberry pi with running protobuf server")
    parser.add_argument("msgs", nargs="+", default=MessageType.keys(),
                        choices=MessageType.keys(), help="List of message types to receive")
    args = parser.parse_args()

    # Set logging format
    FORMAT = '[%(asctime)s %(filename)15s line: %(lineno)5s] %(message)s'
    logging.basicConfig(stream=sys.stdout, format=FORMAT, level=getattr(logging, args.verbosity))
    log = logging.getLogger("protobuf_client")

    # Create a socket connection to ip address and port provided in command line
    c_sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    log.info("Trying to connect to {}:{}".format(args.ip_add, args.port))
    c_sock.connect((args.ip_add, args.port))

    # Create a register request to the server indicating which measurements are requested
    register = Register()
    register.msg_type.extend(map(lambda x: getattr(messages_pb2, x), args.msgs))

    # Serialize to a string before sending
    msg = register.SerializeToString()

    # Calculate length and send as big endian bytes
    length = len(msg)
    log.info("Sending {} as length".format(length))
    c_sock.send(struct.pack(">h", length))

    # Send serialized payload
    log.info("Sending payload")
    c_sock.send(msg)

    # Wait for an ack from the server
    log.info("Waiting for response")
    length_in_bytes = c_sock.recv(2)
    log.info("Recevied a response")
    length = struct.unpack(">h", length_in_bytes)[0]
    log.debug("Trying to receive {} bytes".format(length))
    msg = c_sock.recv(length)
    response = Response()
    response.ParseFromString(msg)
    log.info(response)

    # If the server doesn't acknowledge shutdown
    if response.status == NAK:
        log.info("Received a NAK exiting")
        sys.exit()

    # Loop catching messages
    while True:
        try:
            # Wait for a measurement
            log.info("Waiting for measurement")
            length_in_bytes = c_sock.recv(2)

            # Recv, unpack, and decode generic message
            log.info("Recevied a measurement")
            length = struct.unpack(">h", length_in_bytes)[0]
            log.debug("Trying to receive {} bytes".format(length))
            msg = c_sock.recv(length)
            log.debug("Received {} bytes".format(length))
            generic = Generic()
            generic.ParseFromString(msg)

            # Get what type of measurement it is
            if generic.msg_type == ACCELMEASUREMENT:
                log.info("Message is accel measurement")
                meas = AccelMeasurement()
                accel_list.append(meas)
            elif generic.msg_type == GYROMEASUREMENT:
                log.info("Message is Gyro measurement")
                meas = GyroMeasurement()
                gyro_list.append(meas)
            else:
                log.info("Unknown type continuing")
                continue

            # Decode measurement and print to screen
            meas.ParseFromString(generic.msg)
            for line in str(meas).split("\n"):
                log.info(line)
        except:
            log.info("Stopping")
            break

    if len(accel_list) > 0:
        # Get individual x, y, z measurements for plotting
        accel_x = map(lambda x: x.x_accel, accel_list)
        accel_y = map(lambda x: x.y_accel, accel_list)
        accel_z = map(lambda x: x.z_accel, accel_list)

        # Plot X Accel if there are any
        if len(accel_x):
            plt.figure(1)
            plt.suptitle("X Accel")
            plt.stem(accel_x)

        # Plot Y Accel if there are any
        if len(accel_y):
            plt.figure(2)
            plt.suptitle("Y Accel")
            plt.stem(accel_y)

        # Plot Z Accel if there are any
        if len(accel_z):
            plt.figure(3)
            plt.suptitle("Z Accel")
            plt.stem(accel_z)

    if len(gyro_list) > 0:
        # Get individual x, y, z, temp  measurements for plotting
        gyro_x = map(lambda x: x.x_axis, gyro_list)
        gyro_y = map(lambda x: x.y_axis, gyro_list)
        gyro_z = map(lambda x: x.z_axis, gyro_list)
        gyro_temp = map(lambda x: x.temp, gyro_list)

        # Plot X Gyro if there are any
        if len(gyro_x):
            plt.figure(4)
            plt.suptitle("X Gyro")
            plt.stem(gyro_x)

        # Plot Y Gyro if there are any
        if len(gyro_y):
            plt.figure(5)
            plt.suptitle("Y Gyro")
            plt.stem(gyro_y)

        # Plot Z Gyro if there are any
        if len(gyro_z):
            plt.figure(6)
            plt.suptitle("Z Gyro")
            plt.stem(gyro_z)

        # Plot Temp if there are any
        if len(gyro_temp):
            plt.figure(7)
            plt.suptitle("Temp")
            plt.stem(gyro_temp)


    # Show the plot
    plt.show()
