'''
protobuf_server.py
Script that uses interrupts to grab measurements from SparkFun6DoF then serves them up to
connected clients who have registered for measurements.
ryan.j.mortenson@gmail.com
November 12th, 2016
https://github.com/sparkfun/SparkFun_6DOF_IMU_Pi_Library

Resources:
cffi==1.9.1
pycparser==2.17
smbus-cffi==0.5.1
protobuf==2.6.1

Development environment specifics:
python 2.7
vim 7.4
Raspberry Pi 3
Ubuntu 16.04

This code is released under the [MIT License](http:#opensource.org/licenses/MIT).

Please review the LICENSE.md file included with this example. If you have any questions
or concerns with licensing, please contact techsupport@sparkfun.com.

Distributed as-is; no warranty is given.
'''

import argparse
import RPi.GPIO as gpio
import logging
import socket
import src.adxl345 as Adxl345
import src.itg3200 as Itg3200
import struct
import sys
import threading
import traceback
import messages_pb2
from messages_pb2 import ACCELMEASUREMENT, GYROMEASUREMENT
from messages_pb2 import Generic, GyroMeasurement, AccelMeasurement
from messages_pb2 import Register, Response, ACK, NAK, MessageType
from time import sleep


# Global variables
log = None
gyro_reads = 0
accel_reads = 0
gyro = GyroMeasurement()
accel = AccelMeasurement()
client_list = list()

def send_measurement(meas_type, meas):
    '''
    Sends a measurement via socket to registered clients

    :param meas_type: measurement type as integer value
    :param meas: populate protobuf structure
    '''

    # Create a Generic message and populate fields
    generic = Generic()
    generic.msg_type = meas_type
    generic.msg = meas.SerializeToString()
    msg = generic.SerializeToString()

    # Keep a list of failures to remove from client list
    failures = list()

    # Iterate over clients and check if they are registered for measure type
    for client in client_list:
        if meas_type in client[1]:
            # Get the lenght in bytes
            length_in_bytes = struct.pack(">h", len(msg))
            try:
                # Send length then payload
                client[0].send(length_in_bytes)
                client[0].send(msg)
            except:
                # If an exception occurs add client to failure list
                failures.append(client)

    # Iterate over failures to remove them from client list
    for failure in failures:
        log.info("Removing client: {}".format(failure))
        failure[0].close()
        client_list.remove(failure)

def adxl_callback(channel):
    '''
    Callback function for interrupt on adxl pin
    '''

    log.debug("Interrupt caught for adxl")

    # Read all Accel measurements at once
    global accel, accel_reads
    try:
        # Get all accel measurements
        adxl345.get_all(accel)

        # Read gyro to reset interrupt
        itg3200.read_byte_data(itg3200.INT_STATUS)
        accel_reads += 1
    except:
        # Reading failed so bail
        log.debug("Failed to get measurement")
        log.debug(traceback.format_exc())
        return False

    # Send the measurement via socket
    send_measurement(ACCELMEASUREMENT, accel)

def itg_callback(channel):
    '''
    Callback function for interrupt on itg pin
    '''

    log.debug("Interrupt caught for adxl")

    # Read all Gyro measurements at once
    global gyro, gyro_reads
    try:
        # Get all gyro measurements
        itg3200.get_all(gyro)

        # Read accel to reset interrupt
        adxl345.get_x_accel()
        gyro_reads += 1
    except:
        # Reading failed so bail
        log.debug("Failed to get measurement")
        log.debug(traceback.format_exc())
        return False

    # Send the measurement via socket
    send_measurement(GYROMEASUREMENT, gyro)

if __name__ == "__main__":

    # Setup some arguments for command line execution
    parser = argparse.ArgumentParser()
    parser.add_argument("--i2cbus", "-i", type=int,
                        default=1, help="I2C bus to use default is 1")
    parser.add_argument("--verbosity", "-v", type=str, choices=["DEBUG", "INFO"],
                        default="INFO", help="Repititions default is 1")
    parser.add_argument("--adxl_interrupt", "-a", type=int,
                        default=27, help="Pin adxl interrupt is on")
    parser.add_argument("--itg_interrupt", "-t", type=int,
                        default=17, help="Pin itg interrupt is on")
    args = parser.parse_args()

    # Set logging format
    FORMAT = '[%(asctime)s %(filename)15s line: %(lineno)5s] %(message)s'
    logging.basicConfig(stream=sys.stdout, format=FORMAT, level=getattr(logging, args.verbosity))
    log = logging.getLogger("protobuf_server")

    # Set gpio mode
    gpio.setmode(gpio.BCM)

    # Setup gpio pins as input and set interrupt
    gpio.setup(args.adxl_interrupt, gpio.IN, pull_up_down=gpio.PUD_UP)
    gpio.add_event_detect(args.adxl_interrupt, gpio.FALLING, callback=adxl_callback, bouncetime=3)
    gpio.setup(args.itg_interrupt, gpio.IN, pull_up_down=gpio.PUD_UP)
    gpio.add_event_detect(args.itg_interrupt, gpio.FALLING, callback=itg_callback, bouncetime=3)

    # Instantiate device classes
    itg3200 = Itg3200.Itg3200(args.i2cbus)
    adxl345 = Adxl345.Adxl345(args.i2cbus)

    # Turn on measurement for adxl345
    adxl345.measurement_on()

    # Setup accel for 10-bit mode with no justification at +/- 2g range
    adxl345.write_byte_data(adxl345.DATA_FORMAT, 32)

    # Setup accel with a data rate of 25 Hz
    adxl345.write_byte_data(adxl345.BW_RATE, 8)

    # Set accel INT_ENABLE to 10000000 which is enable DATA_READY
    # Set accel INT_MAP to 00000000 which is all interrupts on INT1 pin
    adxl345.set_interrupt_mode(128, 0)

    # Setup gyro with a sample rate divider of 255 this combined with
    # bandwidth of 256Hz bandwidth gives a data rate of about 32 Hz
    itg3200.write_byte_data(itg3200.SMPLRT_DIV, 255)

    # Setup gyro for +/-2000 deg/sec with low-pass filter bandwidth of 256 Hz
    itg3200.write_byte_data(itg3200.DLPF_FS, 24)

    # Set gyro interrupt mode to 01110101 which is:
    # Active high, open drain, latch until interrupt cleared, clear on any register read
    # enable when device is ready, enable interrupt when data is ready
    itg3200.set_interrupt_mode(33)

    # Read data to start reset interrupts on devices
    adxl345.get_x_accel()
    itg3200.get_x_axis()

    # Loop until users hits ctrl-c then print number of measurements
    try:
        # Get a server socket
        s_sock = None
        s_sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        s_sock.bind(("", 12345))
        s_sock.listen(5)

        while(1):
            # Accept incoming socket connections
            log.info("Accepting incoming socket connections")
            (c_sock, add) = s_sock.accept()
            log.info("Accepted socket connection from address: {}".format(add))

            # Wait for register messages
            log.info("Waiting for Register")

            # Read the first 2 bytes will indicate the payload size
            length_in_bytes = c_sock.recv(2)
            log.debug("Received length bytes")

            # Decode bytes into an integer and use to recv payload bytes
            length = struct.unpack(">h", length_in_bytes)[0]
            log.debug("Length is {}".format(length))
            msg_in_bytes = c_sock.recv(length)

            # Decode register command
            request = Register()
            request.ParseFromString(msg_in_bytes)
            req_msgs = map(lambda x: MessageType._enum_type.values_by_number[x].name, request.msg_type)
            log.info("Received a register for {}".format(req_msgs))

            # Add client to client list
            log.debug("Adding to client list: ")
            client_list.append((c_sock, request.msg_type))

            # Respond with an ack
            log.info("Sending an ACK response")
            response = Response()
            response.status = ACK
            msg = response.SerializeToString()

            # Convert length to bytes
            length = len(msg)
            length_in_bytes = struct.pack(">h", length)

            # Send lenghth in bytes then send message payload
            c_sock.send(length_in_bytes)
            c_sock.send(msg)
            log.info("Sent Response")
    except:
        raise
    finally:
        # Cleanup the gpio
        gpio.cleanup()

        # Close any remaing clients
        for client in client_list:
            client[0].close()

        log.info(s_sock)

        # Close and shutdown receive socket
        if s_sock is not None:
            s_sock.shutdown(socket.SHUT_RDWR)
            s_sock.close()

        # Display how many reads were sent
        log.info("Accel reads: {}".format(accel_reads))
        log.info("Gyro reads: {}".format(gyro_reads))
