'''
Adxl345.py
Adxl345 class definition
ryan.j.mortenson@gmail.com
November 12th, 2016
https://github.com/sparkfun/SparkFun_6DOF_IMU_Pi_Library

Python class for interfacing with the Adxl345 uses i2ccomponent base class

Resources:
cffi==1.9.1
pycparser==2.17
smbus-cffi==0.5.1

Development environment specifics:
python 2.7
vim 7.4
Raspberry Pi 3
Ubuntu 16.04

This code is released under the [MIT License](http:#opensource.org/licenses/MIT).

Please review the LICENSE.md file included with this example. If you have any questions
or concerns with licensing, please contact techsupport@sparkfun.com.

Distributed as-is; no warranty is given.
'''

import logging
from src.i2ccomponent import I2CComponent, entry_func, list_to_int16

def entry_func(func):
    '''
    Decorator for displaying function entry

    :param func: function reference

    :return: wrapper function reference
    '''

    def wrapper(self, *args, **kwargs):
        '''
        Wrapper around functions to display entry

        :return: original function reference
        '''

        # Display function entry and return original function reference
        self.log.debug("Entering function {}".format(func.__name__))
        return func(self, *args, **kwargs)

    return wrapper

class Adxl345(I2CComponent):

    # Constants for reading measurements
    MEAS_SIZE = 2
    ALL_MEAS_SIZE = 3*2

    # Constants for turning measurements on and off
    MEAS_ON = 1<<3
    MEAS_OFF = 0xff ^ 1<<3

    # Format used for struct.unpack
    FMT = ">h"

    def __init__(self, bus):
        '''
        Constructor

        :param bus: bus i2c is on
        '''

        self.log = logging.getLogger("Adxl345")
        self.add = 0x53
        self.x_accel = 0
        self.y_accel = 0
        self.z_accel = 0

        super(Adxl345, self).__init__(bus)

    @entry_func
    def measurement_on(self):
        '''
        Sets the measurement on for Adxl345
        '''

        # Get the current POWER_CTL status
        status = self.read_byte_data(Adxl345.POWER_CTL)

        # Write new status with MEAS_ON
        self.write_byte_data(Adxl345.POWER_CTL, self.MEAS_ON | status)

    @entry_func
    def measurement_off(self):
        '''
        Sets the measurement off for Adxl345
        '''

        # Get the current POWER_CTL status
        status = self.read_byte_data(Adxl345.POWER_CTL)

        # Write new status with MEAS_ON
        self.write_byte_data(Adxl345.POWER_CTL, self.MEAS_OFF & status)

    @entry_func
    def set_interrupt_mode(self, enable, map):
        '''
        Sets the interrept mode for Adxl345

        :param mode: mode to set the interrupt (check data sheet)
        '''

        # Write to interrupt enable register
        self.write_byte_data(Adxl345.INT_ENABLE, enable)

        # Write the interrupt map
        self.write_byte_data(Adxl345.INT_MAP, map)

    @entry_func
    def get_interrupt_enable(self):
        '''
        Get the interrept enable register for Adxl345

        :return: interrupt enable register value
        '''

        return self.read_byte_data(Adxl345.INT_ENABLE)

    @entry_func
    def get_interrupt_map(self):
        '''
        Get the interrept map register for Adxl345

        :return: interrupt map register value
        '''

        return self.read_byte_data(Adxl345.INT_MAP)

    @entry_func
    def get_x_accel(self):
        '''
        Get the x accel measurement from the Adxl345

        :return: x accel measurement as integer
        '''

        ret = self.read_i2c_block_data(Adxl345.DATAX0,
                                       Adxl345.MEAS_SIZE)
        return list_to_int16(ret, Adxl345.FMT)

    @entry_func
    def get_y_accel(self):
        '''
        Get the y accel measurement from the Adxl345

        :return: y accel measurement as integer
        '''

        ret = self.read_i2c_block_data(Adxl345.DATAY0,
                                       Adxl345.MEAS_SIZE)
        return list_to_int16(ret, Adxl345.FMT)

    @entry_func
    def get_z_accel(self):
        '''
        Get the z accel measurement from the Adxl345

        :return: z accel measurement as integer
        '''

        ret = self.read_i2c_block_data(Adxl345.DATAZ0,
                                       Adxl345.MEAS_SIZE)
        return list_to_int16(ret, Adxl345.FMT)

    @entry_func
    def get_all(self, struct=None):
        '''
        Get the all of the measurements from the Adxl345

        :param struct: structure to fill out if none fill out self

        :return: list of values
        '''

        # Read all the measurement registers
        ret = self.read_i2c_block_data(Adxl345.DATAX0,
                                       Adxl345.ALL_MEAS_SIZE)

        # If a structure isn't provided use self
        if struct is None:
            struct = self

        struct.x_accel = list_to_int16(ret[0:2], Adxl345.FMT)
        struct.y_accel = list_to_int16(ret[2:4], Adxl345.FMT)
        struct.z_accel = list_to_int16(ret[4:6], Adxl345.FMT)

        return ret

    '''
    Register values for the Adxl345 consult datasheet for description
    '''

    # Device Id
    DEVID = 0x00

    # Tap Threshold
    THRESH_TAP = 0x1d

    # User defined X, Y, and Z offsets
    OFSX = 0x1e
    OFSY = 0x1f
    OFSZ = 0x20

    # Tap duration
    DUR = 0x21

    # Tap latency
    LATENT = 0x22

    # Tap window
    WINDOW = 0x23

    # Activity and Inactivity thresholds
    THRESH_ACT = 0x24
    THRESH_INACT = 0x25

    # Inactivity time
    TIME_INACT = 0x26

    # Axis enable control for activity and inactivity detection
    ACT_INACT_CTL = 0x27

    # Free-fall threshold and time
    THRESH_FF = 0x28
    TIME_FF = 0x29

    # Axis control for tap/double tap
    TAP_AXES = 0x2a

    # Source of tap/double tap
    ACT_TAP_STATUS = 0x2b

    # Data rate and power mode
    BW_RATE = 0x2c

    # Power saving control
    POWER_CTL = 0x2d

    # Interrupt enable, amp, and source
    INT_ENABLE = 0x2e
    INT_MAP = 0x2f
    INT_SOURCE = 0x30

    # Data format control
    DATA_FORMAT = 0x31

    # X, Y, and Z accleration
    DATAX0 = 0x32
    DATAX1 = 0x33
    DATAY0 = 0x34
    DATAY1 = 0x35
    DATAZ0 = 0x36
    DATAZ1 = 0x37

    # FIFO control and status
    FIFO_CTL = 0x38
    FIFO_STATUS = 0x39
