'''
i2ccomponent.py
I2CComponent base class
ryan.j.mortenson@gmail.com
November 12th, 2016
https://github.com/sparkfun/SparkFun_6DOF_IMU_Pi_Library

Python class interfacing with i2c components.

Resources:
cffi==1.9.1
pycparser==2.17
smbus-cffi==0.5.1

Development environment specifics:
python 2.7
vim 7.4
Raspberry Pi 3
Ubuntu 16.04

This code is released under the [MIT License](http:#opensource.org/licenses/MIT).

Please review the LICENSE.md file included with this example. If you have any questions
or concerns with licensing, please contact techsupport@sparkfun.com.

Distributed as-is; no warranty is given.
'''

import smbus
import logging
import struct
import array

def entry_func(func):
    '''
    Decorator for displaying function entry

    :param func: function reference

    :return: wrapper function reference
    '''

    def wrapper(self, *args, **kwargs):
        '''
        Wrapper around functions to display entry

        :return: original function reference
        '''

        # Display function entry and return original function reference
        self.log.debug("Entering function {}".format(func.__name__))
        return func(self, *args, **kwargs)

    return wrapper

def list_to_int16(val_list, fmt):
    '''
    Convert list of 2 bytes to int16

    :param val_list: list of 2 byte values
    :param fmt: struct module format for endianess

    :return: 16 bit integer
    '''

    return struct.unpack(fmt, array.array("B", val_list).tostring())[0]

class I2CComponent(smbus.SMBus):

    def __init__(self, bus):
        '''
        Constructor

        :param bus: bus i2c is on
        '''

        self.log = logging.getLogger("I2CComponent")
        super(I2CComponent, self).__init__(bus)

    @entry_func
    def write_byte_data(self, cmd, val):
        '''
        Overloaded write_byte_data from smbus to add logging

        :param cmd: register to write
        :param val: value to write to register

        :return: return value from base class call to write_byte_data
        '''

        msg = "Writing {} to register {} at address {}"
        self.log.debug(msg.format(val, hex(cmd), hex(self.add)))
        return super(I2CComponent, self).write_byte_data(self.add, cmd, val)

    @entry_func
    def read_byte_data(self, cmd):
        '''
        Overloaded read_byte_data from smbus to add logging

        :param cmd: register to read

        :return: return value from base class call to write_byte_data
        '''

        msg = "Reading register {} from address {} returned {}"
        ret = super(I2CComponent, self).read_byte_data(self.add, cmd)
        self.log.debug(msg.format(hex(cmd), hex(self.add), ret))
        return ret

    @entry_func
    def read_i2c_block_data(self, cmd, len):
        '''
        Overloaded read_byte_data from smbus to add logging

        :param cmd: register to read
        :param len: number of registers to read

        :return: return value from base class call to write_byte_data
        '''

        msg = "Reading {} bytes from  register {} from address {} returned {}"
        ret = super(I2CComponent, self).read_i2c_block_data(self.add, cmd, len)
        self.log.debug(msg.format(len, hex(cmd), hex(self.add), ret))
        return ret
