'''
Itg3200.py
Itg3200 class definition
ryan.j.mortenson@gmail.com
November 12th, 2016
https://github.com/sparkfun/SparkFun_6DOF_IMU_Pi_Library

Python class for interfacing with the Itg3200 uses i2ccomponent base class

Resources:
cffi==1.9.1
pycparser==2.17
smbus-cffi==0.5.1

Development environment specifics:
python 2.7
vim 7.4
Raspberry Pi 3
Ubuntu 16.04

This code is released under the [MIT License](http:#opensource.org/licenses/MIT).

Please review the LICENSE.md file included with this example. If you have any questions
or concerns with licensing, please contact techsupport@sparkfun.com.

Distributed as-is; no warranty is given.
'''

import logging
from src.i2ccomponent import I2CComponent, entry_func, list_to_int16

def entry_func(func):
    '''
    Decorator for displaying function entry

    :param func: function reference

    :return: wrapper function reference
    '''

    def wrapper(self, *args, **kwargs):
        '''
        Wrapper around functions to display entry

        :return: original function reference
        '''

        # Display function entry and return original function reference
        self.log.debug("Entering function {}".format(func.__name__))
        return func(self, *args, **kwargs)

    return wrapper

class Itg3200(I2CComponent):

    # Constants for reading measurements
    MEAS_SIZE = 2
    ALL_MEAS_SIZE = 4*2

    # Format used for struct.unpack
    FMT = ">h"

    def __init__(self, bus):
        '''
        Constructor

        :param bus: bus i2c is on
        '''

        self.log = logging.getLogger("Itg3200")
        self.add = 0x68
        self.x_axis = 0
        self.y_axis = 0
        self.z_axis = 0
        self.temp = 0

        super(Itg3200, self).__init__(bus)

    @entry_func
    def set_interrupt_mode(self, mode):
        '''
        Sets the interrept mode for Itg3200

        :param mode: mode to set the interrupt (check data sheet)
        '''

        return self.write_byte_data(Itg3200.INT_CFG, mode)

    @entry_func
    def get_interrupt_mode(self):
        '''
        Get the interrept mode for Itg3200

        :return: interrupt mode
        '''

        return self.read_byte_data(Itg3200.INT_CFG)

    @entry_func
    def get_interrupt_status(self):
        '''
        Get the interrept mode for Itg3200

        :return: interrupt mode
        '''

        return self.read_byte_data(Itg3200.INT_STATUS)

    @entry_func
    def get_x_axis(self):
        '''
        Get the x axis measurement from the Itg3200

        :return: x axis measurement as integer
        '''

        ret = self.read_i2c_block_data(Itg3200.GYRO_XOUT_H,
                                       Itg3200.MEAS_SIZE)
        return list_to_int16(ret, Itg3200.FMT)

    @entry_func
    def get_y_axis(self):
        '''
        Get the y axis measurement from the Itg3200

        :return: y axis measurement as integer
        '''

        ret = self.read_i2c_block_data(Itg3200.GYRO_YOUT_H,
                                       Itg3200.MEAS_SIZE)
        return list_to_int16(ret, Itg3200.FMT)

    @entry_func
    def get_z_axis(self):
        '''
        Get the z axis measurement from the Itg3200

        :return: z axis measurement as integer
        '''

        ret = self.read_i2c_block_data(Itg3200.GYRO_ZOUT_H,
                                       Itg3200.MEAS_SIZE)
        return list_to_int16(ret, Itg3200.FMT)

    @entry_func
    def get_temp(self):
        '''
        Get the temp axis measurement from the Itg3200

        :return: temp axis measurement as integer
        '''

        ret = self.read_i2c_block_data(Itg3200.TEMP_OUT_H,
                                       Itg3200.MEAS_SIZE)
        return list_to_int16(ret, Itg3200.FMT)

    @entry_func
    def get_all(self, struct=None):
        '''
        Get the all of the measurements from the Itg3200

        :param struct: structure to fill out if none fill out self

        :return: list of values
        '''

        ret = self.read_i2c_block_data(Itg3200.TEMP_OUT_H,
                                       Itg3200.ALL_MEAS_SIZE)
        if struct is None:
            struct = self

        struct.temp = list_to_int16(ret[0:2], Itg3200.FMT)
        struct.x_axis = list_to_int16(ret[2:4], Itg3200.FMT)
        struct.y_axis = list_to_int16(ret[4:6], Itg3200.FMT)
        struct.z_axis = list_to_int16(ret[6:8], Itg3200.FMT)

        return ret

    '''
    Register values for the Itg3200 consult datasheet for description
    '''

    # Device ID
    WHO_AM_I = 0

    # Sample Rate Divider
    SMPLRT_DIV = 0x15

    # Digital Low Pass Filter Full Scale
    DLPF_FS = 0x16

    # Interrupt configuration
    INT_CFG = 0x17

    # Interrupt status
    INT_STATUS = 0x1a

    # Temperature high and low bytes
    TEMP_OUT_H = 0x1b
    TEMP_OUT_L = 0x1c
    # X axis high and low bytes
    GYRO_XOUT_H = 0x1d
    GYRO_XOUT_L = 0x1e

    # Y axis high and low bytes
    GYRO_YOUT_H = 0x1f
    GYRO_YOUT_L = 0x20

    # Z axis high and low bytes
    GYRO_ZOUT_H = 0x21
    GYRO_ZOUT_L = 0x22
